#ifndef MSP_GLTK_DROPDOWN_H_
#define MSP_GLTK_DROPDOWN_H_

#include <sigc++/signal.h>
#include "list.h"
#include "text.h"
#include "widget.h"

namespace Msp {
namespace GLtk {

class List;

class Dropdown: virtual public Widget, private Container
{
public:
	class Loader: public DataFile::DerivedObjectLoader<Dropdown, Widget::Loader>
	{
	public:
		Loader(Dropdown &);
	private:
		void item(const std::string &);
	};

	sigc::signal<void, unsigned> signal_item_selected;

private:
	List list;
	bool dropped;
	Text text;

public:
	Dropdown();
	Dropdown(ListData &);
private:
	void init();

public:
	virtual const char *get_class() const { return "dropdown"; }

private:
	virtual void autosize_special(const Part &, Geometry &) const;

public:
	void set_data(ListData &d) { list.set_data(d); }
	ListData &get_data() { return list.get_data(); }
	const ListData &get_data() const { return list.get_data(); }

	template<typename T>
	void set_item_type() { list.set_item_type<T>(); }

	void set_selected_index(int i) { list.set_selected_index(i); }
	int get_selected_index() const { return list.get_selected_index(); }

private:
	virtual void rebuild_special(const Part &);
	virtual void render_special(const Part &, GL::Renderer &) const;

public:
	virtual void button_press(int, int, unsigned);
private:
	virtual void on_geometry_change();
	virtual void on_style_change();

	void list_autosize_changed();
	void resize_list();
	void list_item_selected(unsigned);
	void list_selection_cleared();
};

} // namespace GLtk
} // namespace Msp

#endif
