#ifndef MSP_GLTK_USERINTERFACE_H_
#define MSP_GLTK_USERINTERFACE_H_

#include <msp/core/maputils.h>
#include "logic.h"
#include "root.h"

namespace Msp {
namespace GLtk {

/**
Encapsulates a Root widget and Logic associated with it.  Allows looking up
widgets by name.
*/
class UserInterface
{
public:
	class Loader: public DataFile::Loader
	{
	private:
		UserInterface &ui;

	public:
		Loader(UserInterface &);
	private:
		void logic();
		void root();
	};

	typedef std::map<std::string, Widget *> WidgetMap;

private:
	WidgetMap widgets;
	Root root;
	Logic logic;

public:
	UserInterface(Resources &, Graphics::Window &);
	Root &get_root() { return root; }
	const Logic &get_logic() const { return logic; }

	template<typename W>
	W &get_widget(const std::string &n) const
	{
		return dynamic_cast<W &>(*get_item(widgets, n));
	}

	template<typename W>
	void get_widget(const std::string &n, W *&w) const
	{
		w = &get_widget<W>(n);
	}
};

} // namespace GLtk
} // namespace Msp

#endif
